@@ -1,3 +1,4 @@
+"""Main package"""
 import os
 import pytest
 import pkg_resources
@@ -5,6 +6,7 @@ from .region import Region
 from .mesh import Mesh
 from .field import Field
 from .line import Line
+from .operators import DValue, dx, dy, dz, dV, dS, integral
 from .interact import interact
 import matplotlib.pyplot as plt
 
@@ -18,5 +20,16 @@ __dependencies__ = pkg_resources.require(__name__)
 
 
 def test():
+    """Run all package tests.
+
+    Examples
+    --------
+    1. Run all tests.
+
+    >>> import discretisedfield as df
+    ...
+    >>> # df.test()
+
+    """
     return pytest.main(['-v', '--pyargs',
                         'discretisedfield', '-l'])  # pragma: no cover
