@@ -63,9 +63,6 @@ def check_field(field):
     assert 1*field == field
     assert -1*field == -field
 
-    integral = field.volume_integral
-    assert isinstance(integral, (tuple, numbers.Real))
-
     if field.dim == 1:
         grad = field.grad
         assert isinstance(grad, df.Field)
@@ -73,6 +70,14 @@ def check_field(field):
 
         assert all(i not in dir(field) for i in 'xyz')
 
+        assert isinstance((field * df.dx).integral(), numbers.Real)
+        assert isinstance((field * df.dy).integral(), numbers.Real)
+        assert isinstance((field * df.dz).integral(), numbers.Real)
+        assert isinstance((field * df.dV).integral(), numbers.Real)
+        assert isinstance((field.plane('z') * df.dS).integral(), tuple)
+        assert isinstance((field.plane('z') * abs(df.dS)).integral(),
+                          numbers.Real)
+
     if field.dim == 3:
         norm = field.norm
         assert isinstance(norm, df.Field)
@@ -97,9 +102,13 @@ def check_field(field):
         assert curl.dim == 3
 
         field_plane = field.plane('z')
-        assert isinstance(field_plane.topological_charge_density, df.Field)
-        assert isinstance(field_plane.topological_charge(), numbers.Real)
-        assert isinstance(field_plane.bergluescher, numbers.Real)
+
+        assert isinstance((field * df.dx).integral(), tuple)
+        assert isinstance((field * df.dy).integral(), tuple)
+        assert isinstance((field * df.dz).integral(), tuple)
+        assert isinstance((field * df.dV).integral(), tuple)
+        assert isinstance((field.plane('z') @ df.dS).integral(), numbers.Real)
+        assert isinstance((field.plane('z') * abs(df.dS)).integral(), tuple)
 
         orientation = field.orientation
         assert isinstance(orientation, df.Field)
@@ -217,8 +226,8 @@ class TestField:
         p1 = (0, 0, 0)
         p2 = (10e-9, 10e-9, 10e-9)
         n = (5, 5, 5)
-        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(5e-9, 10e-9, 10e-9)),
-                      'r2': df.Region(p1=(5e-9, 0, 0),
+        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(4e-9, 10e-9, 10e-9)),
+                      'r2': df.Region(p1=(4e-9, 0, 0),
                                       p2=(10e-9, 10e-9, 10e-9))}
         mesh = df.Mesh(p1=p1, p2=p2, n=n, subregions=subregions)
 
@@ -285,8 +294,6 @@ class TestField:
         # Exception
         mesh = df.Mesh(p1=(0, 0, 0), p2=(10, 10, 10), cell=(1, 1, 1))
         f = df.Field(mesh, dim=1, value=-5)
-        with pytest.raises(ValueError):
-            norm = f.norm
         with pytest.raises(ValueError):
             f.norm = 5
 
@@ -833,7 +840,7 @@ class TestField:
         with pytest.raises(ValueError):
             res = f1 & f2
 
-    def test_stack(self):
+    def test_lshift(self):
         p1 = (0, 0, 0)
         p2 = (10e6, 10e6, 10e6)
         cell = (5e6, 5e6, 5e6)
@@ -1298,20 +1305,24 @@ class TestField:
 
         assert f.laplace.average == (4, 4, 6)
 
-    def test_volume_integral(self):
+    def test_integral(self):
+        # Volume integral.
         p1 = (0, 0, 0)
         p2 = (10, 10, 10)
         cell = (1, 1, 1)
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
 
         f = df.Field(mesh, dim=1, value=0)
-        assert f.volume_integral == 0
+        assert (f * df.dV).integral() == 0
+        assert (f * df.dx*df.dy*df.dz).integral() == 0
 
         f = df.Field(mesh, dim=1, value=2)
-        assert f.volume_integral == 2000
+        assert (f * df.dV).integral() == 2000
+        assert (f * df.dx*df.dy*df.dz).integral() == 2000
 
         f = df.Field(mesh, dim=3, value=(-1, 0, 3))
-        assert f.volume_integral == (-1000, 0, 3000)
+        assert (f * df.dV).integral() == (-1000, 0, 3000)
+        assert (f * df.dx*df.dy*df.dz).integral() == (-1000, 0, 3000)
 
         def value_fun(point):
             x, y, z = point
@@ -1321,71 +1332,82 @@ class TestField:
                 return (1, 2, 3)
 
         f = df.Field(mesh, dim=3, value=value_fun)
-        assert f.volume_integral == (0, 0, 0)
+        assert (f * df.dV).integral() == (0, 0, 0)
+        assert (f * df.dx*df.dy*df.dz).integral() == (0, 0, 0)
 
-    def test_surface_integral(self):
+        # Surface integral.
         p1 = (0, 0, 0)
         p2 = (10, 5, 3)
         cell = (1, 1, 1)
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
 
         f = df.Field(mesh, dim=1, value=0)
-        assert f.plane('x').surface_integral == 0
+        assert (f.plane('x') * abs(df.dS)).integral() == 0
+        assert (f.plane('x') * df.dy*df.dz).integral() == 0
 
         f = df.Field(mesh, dim=1, value=2)
-        assert f.plane('x').surface_integral == 30
-        assert f.plane('y').surface_integral == 60
-        assert f.plane('z').surface_integral == 100
+        assert (f.plane('x') * abs(df.dS)).integral() == 30
+        assert (f.plane('x') * df.dy*df.dz).integral() == 30
+        assert (f.plane('y') * abs(df.dS)).integral() == 60
+        assert (f.plane('y') * df.dx*df.dz).integral() == 60
+        assert (f.plane('z') * abs(df.dS)).integral() == 100
+        assert (f.plane('z') * df.dx*df.dy).integral() == 100
 
         f = df.Field(mesh, dim=3, value=(-1, 0, 3))
-        assert f.plane('x').surface_integral == (-15, 0, 45)
-        assert f.plane('y').surface_integral == (-30, 0, 90)
-        assert f.plane('z').surface_integral == (-50, 0, 150)
+        assert (f.plane('x') * abs(df.dS)).integral() == (-15, 0, 45)
+        assert (f.plane('y') * abs(df.dS)).integral() == (-30, 0, 90)
+        assert (f.plane('z') * abs(df.dS)).integral() == (-50, 0, 150)
 
-    def test_topological_charge(self):
+        f = df.Field(mesh, dim=3, value=(-1, 0, 3))
+        assert df.integral(f.plane('x') @ df.dS) == -15
+        assert df.integral(f.plane('y') @ df.dS) == 0
+        assert df.integral(f.plane('z') @ df.dS) == 150
+
+        # Directional integral
         p1 = (0, 0, 0)
         p2 = (10, 10, 10)
-        cell = (2, 2, 2)
+        cell = (1, 1, 1)
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+        f = df.Field(mesh, dim=3, value=(1, 1, 1))
 
-        # f(x, y, z) = (0, 0, 0)
-        # -> Q(f) = 0
-        f = df.Field(mesh, dim=3, value=(0, 0, 0))
+        f = f.integral(direction='x')
+        assert isinstance(f, df.Field)
+        assert f.dim == 3
+        assert f.mesh.n == (1, 10, 10)
+        assert f.average == (10, 10, 10)
 
-        q = f.plane('z').topological_charge_density
-        check_field(q)
-        assert q.dim == 1
-        assert q.average == 0
-        assert f.plane('z').topological_charge(method='continuous') == 0
-        assert f.plane('z').topological_charge(method='berg-luescher') == 0
+        f = f.integral(direction='x').integral(direction='y')
+        assert isinstance(f, df.Field)
+        assert f.dim == 3
+        assert f.mesh.n == (1, 1, 10)
+        assert f.average == (100, 100, 100)
 
-        # Skyrmion from a file
-        test_filename = os.path.join(os.path.dirname(__file__),
-                                     'test_sample/',
-                                     'skyrmion.omf')
-        f = df.Field.fromfile(test_filename)
-        Qc = f.plane('z').topological_charge(method='continuous')
-        Qbl = f.plane('z').topological_charge(method='berg-luescher')
-        assert abs(Qc - 1) < 0.15
-        assert abs(Qbl - 1) < 1e-3
+        f = f.integral('x').integral('y').integral('z')
+        assert f.dim == 3
+        assert f.mesh.n == (1, 1, 1)
+        assert f.average == (1000, 1000, 1000)
 
-        # Not sliced
-        f = df.Field(mesh, dim=3, value=(1, 2, 3))
+        assert (f.integral('x').integral('y').integral('z').average ==
+                f.integral())
+
+        # Improper integral
+        p1 = (0, 0, 0)
+        p2 = (10, 10, 10)
+        cell = (1, 1, 1)
+        mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+        f = df.Field(mesh, dim=3, value=(1, 1, 1))
+
+        f = f.integral(direction='x', improper=True)
+        assert isinstance(f, df.Field)
+        assert f.dim == 3
+        assert f.mesh.n == (10, 10, 10)
+        assert f.average == (5.5, 5.5, 5.5)
+        assert f((0, 0, 0)) == (1, 1, 1)
+        assert f((10, 10, 10)) == (10, 10, 10)
+
+        # Exceptions
         with pytest.raises(ValueError):
-            res = f.topological_charge(method='continuous')
-        with pytest.raises(ValueError):
-            res = f.topological_charge(method='berg-luescher')
-
-        # Scalar field
-        f = df.Field(mesh, dim=1, value=3.14)
-        with pytest.raises(ValueError):
-            res = f.plane('z').topological_charge(method='continuous')
-        with pytest.raises(ValueError):
-            res = f.plane('z').topological_charge(method='berg-luescher')
-
-        # Method does not exist
-        with pytest.raises(ValueError):
-            res = f.plane('z').topological_charge(method='some-method')
+            res = f.integral(direction='xy', improper=True)
 
     def test_line(self):
         mesh = df.Mesh(p1=(0, 0, 0), p2=(10, 10, 10), n=(10, 10, 10))
@@ -1428,12 +1450,21 @@ class TestField:
         check_field(f)
         check_field(f['r1'])
         check_field(f['r2'])
+        check_field(f[subregions['r1']])
+        check_field(f[subregions['r2']])
 
         assert f['r1'].average == (-1, -2, -3)
         assert f['r2'].average == (0, 0, 0)
+        assert f[subregions['r1']].average == (-1, -2, -3)
+        assert f[subregions['r2']].average == (0, 0, 0)
 
         assert len(f['r1'].mesh) + len(f['r2'].mesh) == len(f.mesh)
 
+        # Meshes are not aligned
+        subregion = df.Region(p1=(1.1, 0, 0), p2=(9.9, 15, 5))
+
+        assert f[subregion].array.shape == (2, 3, 1, 3)
+
     def test_project(self):
         p1 = (-5, -5, -5)
         p2 = (5, 5, 5)
@@ -1499,8 +1530,8 @@ class TestField:
         f = df.Field(mesh, dim=3, value=value_fun)
 
         assert abs(f.plane('z').angle((1e-9, 2e-9, 2e-9)) - np.pi/4) < 1e-3
-        assert abs(f.plane('z').angle((3e-9, 2e-9, 2e-9)) + np.pi/4) < 1e-3
-        assert abs(f.plane('z').angle((5e-9, 2e-9, 2e-9)) + 3*np.pi/4) < 1e-3
+        assert abs(f.plane('z').angle((3e-9, 2e-9, 2e-9)) - 7*np.pi/4) < 1e-3
+        assert abs(f.plane('z').angle((5e-9, 2e-9, 2e-9)) - 5*np.pi/4) < 1e-3
         assert abs(f.plane('z').angle((7e-9, 2e-9, 2e-9)) - 3*np.pi/4) < 1e-3
 
         # Exception
@@ -1644,6 +1675,18 @@ class TestField:
                                         multiplier=1e-6, cmap='hsv',
                                         clim=(-1, 1))
 
+        # Lightness field
+        filenames = ['skyrmion.omf', 'skyrmion-disk.omf']
+        for i in filenames:
+            filename = os.path.join(os.path.dirname(__file__),
+                                    'test_sample', i)
+
+            field = df.Field.fromfile(filename)
+            field.plane('z').angle.mpl_scalar(lightness_field=field.z)
+            field.plane('z').angle.mpl_scalar(lightness_field=-field.z,
+                                              filter_field=field.norm)
+            field.plane('z').mpl(scalar_lightness_field=-field.z)
+
         # Saving plot
         filename = 'testfigure.pdf'
         with tempfile.TemporaryDirectory() as tmpdir:
@@ -1651,13 +1694,16 @@ class TestField:
             self.pf.x.plane('x', n=(3, 4)).mpl_scalar(filename=tmpfilename)
 
         # Exceptions
-        with pytest.raises(ValueError) as excinfo:
+        with pytest.raises(ValueError):
             self.pf.x.mpl_scalar()  # not sliced
-        with pytest.raises(ValueError) as excinfo:
+        with pytest.raises(ValueError):
             self.pf.plane('z').mpl_scalar()  # vector field
-        with pytest.raises(ValueError) as excinfo:
+        with pytest.raises(ValueError):
             # wrong filter field
             self.pf.x.plane('z').mpl_scalar(filter_field=self.pf)
+        with pytest.raises(ValueError):
+            # wrong filter field
+            self.pf.x.plane('z').mpl_scalar(lightness_field=self.pf)
 
         plt.close('all')
 
