@@ -61,6 +61,8 @@ def check_mesh(mesh):
     assert mesh == mesh
     assert not mesh != mesh
 
+    assert mesh | mesh
+
 
 class TestMesh:
     def setup(self):
@@ -139,6 +141,27 @@ class TestMesh:
         mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
         check_mesh(mesh)
 
+        # Invalid subregions.
+        p1 = (0, 0, 0)
+        p2 = (100e-9, 50e-9, 10e-9)
+        cell = (10e-9, 10e-9, 10e-9)
+
+        # Subregion not an aggregate.
+        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(45e-9, 50e-9, 10e-9))}
+        with pytest.raises(ValueError):
+            mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
+
+        # Subregion not aligned.
+        subregions = {'r1': df.Region(p1=(5e-9, 0, 0),
+                                      p2=(45e-9, 50e-9, 10e-9))}
+        with pytest.raises(ValueError):
+            mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
+
+        # Subregion not in the mesh region.
+        subregions = {'r1': df.Region(p1=(0, 0, 0), p2=(45e-9, 50e-9, 200e-9))}
+        with pytest.raises(ValueError):
+            mesh = df.Mesh(p1=p1, p2=p2, cell=cell, subregions=subregions)
+
     def test_init_with_region_and_points(self):
         p1 = (0, -4, 16.5)
         p2 = (15, 10.1, 11)
@@ -416,7 +439,7 @@ class TestMesh:
             assert len(point) == 3
             assert all([0 <= i <= 10 for i in point])
 
-        line = list(mesh.line((0, 0, 0), (10, 0, 0), n=11))
+        line = list(mesh.line(p1=(0, 0, 0), p2=(10, 0, 0), n=11))
         assert len(line) == 11
         assert line[0] == (0, 0, 0)
         assert line[-1] == (10, 0, 0)
@@ -546,7 +569,34 @@ class TestMesh:
         with pytest.raises(ValueError):
             plane_mesh = mesh.plane('z', x=5)
 
+    def test_or(self):
+        p1 = (-50e-9, -25e-9, 0)
+        p2 = (50e-9, 25e-9, 5e-9)
+        cell = (5e-9, 5e-9, 5e-9)
+        mesh1 = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        p1 = (-45e-9, -20e-9, 0)
+        p2 = (10e-9, 20e-9, 5e-9)
+        cell = (5e-9, 5e-9, 5e-9)
+        mesh2 = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        p1 = (-42e-9, -20e-9, 0)
+        p2 = (13e-9, 20e-9, 5e-9)
+        cell = (5e-9, 5e-9, 5e-9)
+        mesh3 = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        p1 = (-50e-9, -25e-9, 0)
+        p2 = (50e-9, 25e-9, 5e-9)
+        cell = (2.5e-9, 2.5e-9, 2.5e-9)
+        mesh4 = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        assert mesh1 | mesh2 is True
+        assert mesh1 | mesh3 is False
+        assert mesh1 | mesh4 is False
+        assert mesh1 | mesh1 is True
+
     def test_getitem(self):
+        # Subregions disctionary
         p1 = (0, 0, 0)
         p2 = (100, 50, 10)
         cell = (5, 5, 5)
@@ -569,6 +619,42 @@ class TestMesh:
 
         assert len(submesh1) + len(submesh2) == len(mesh)
 
+        # "newly-defined" region
+        p1 = (0, 0, 0)
+        p2 = (10, 10, 10)
+        cell = (1, 1, 1)
+        mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+        check_mesh(mesh)
+
+        submesh = mesh[df.Region(p1=(0.1, 2.2, 4.01), p2=(4.9, 3.8, 5.7))]
+        check_mesh(submesh)
+        assert submesh.region.pmin == (0, 2, 4)
+        assert submesh.region.pmax == (5, 4, 6)
+        assert submesh.cell == cell
+        assert submesh.n == (5, 2, 2)
+        assert mesh[mesh.region] == mesh
+
+        p1 = (20e-9, 0, 15e-9)
+        p2 = (-25e-9, 100e-9, -5e-9)
+        cell = (5e-9, 5e-9, 5e-9)
+        mesh = df.Mesh(p1=p1, p2=p2, cell=cell)
+        check_mesh(mesh)
+
+        submesh = mesh[df.Region(p1=(11e-9, 22e-9, 1e-9),
+                                 p2=(-9e-9, 79e-9, 14e-9))]
+        check_mesh(submesh)
+        assert np.allclose(submesh.region.pmin, (-10e-9, 20e-9, 0),
+                           atol=1e-15, rtol=1e-5)
+        assert np.allclose(submesh.region.pmax, (15e-9, 80e-9, 15e-9),
+                           atol=1e-15, rtol=1e-5)
+        assert submesh.cell == cell
+        assert submesh.n == (5, 12, 3)
+        assert mesh[mesh.region] == mesh
+
+        with pytest.raises(ValueError):
+            submesh = mesh[df.Region(p1=(11e-9, 22e-9, 1e-9),
+                                     p2=(200e-9, 79e-9, 14e-9))]
+
     def test_pad(self):
         p1 = (-1, 2, 7)
         p2 = (5, 9, 4)
@@ -596,6 +682,49 @@ class TestMesh:
         assert padded_mesh.region.pmax == (6, 10, 8)
         assert padded_mesh.n == (8, 9, 5)
 
+    def test_getattr(self):
+        p1 = (0, 0, 0)
+        p2 = (100e-9, 80e-9, 10e-9)
+        cell = (1e-9, 5e-9, 10e-9)
+        mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        assert mesh.dx == 1e-9
+        assert mesh.dy == 5e-9
+        assert mesh.dz == 10e-9
+
+        with pytest.raises(AttributeError):
+            res = mesh.dk
+
+    def test_dir(self):
+        p1 = (0, 0, 0)
+        p2 = (100e-9, 80e-9, 10e-9)
+        cell = (1e-9, 5e-9, 10e-9)
+        mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        assert all([i in dir(mesh) for i in ['dx', 'dy', 'dz']])
+
+    def test_dV(self):
+        p1 = (0, 0, 0)
+        p2 = (100, 80, 10)
+        cell = (1, 2, 2.5)
+        mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        assert mesh.dV == 5
+
+    def test_dS(self):
+        p1 = (0, 0, 0)
+        p2 = (100, 80, 10)
+        cell = (1, 2, 2.5)
+        mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), cell=cell)
+
+        assert mesh.plane('x').dS.average == (5, 0, 0)
+        assert mesh.plane('y').dS.average == (0, 2.5, 0)
+        assert mesh.plane('z').dS.average == (0, 0, 2)
+
+        # Exception
+        with pytest.raises(ValueError):
+            res = mesh.dS
+
     def test_mpl(self):
         for p1, p2, n, cell in self.valid_args:
             mesh = df.Mesh(region=df.Region(p1=p1, p2=p2), n=n, cell=cell)
