@@ -1,3 +1,3 @@
 from .util import axesdict, raxesdict, cp_int, cp_hex, array2tuple, as_array, \
     bergluescher_angle, assemble_index, plot_line, plot_box, vtk_scalar_data, \
-    vtk_vector_data, normalise_to_range
+    vtk_vector_data, normalise_to_range, hls2rgb
